#!/usr/bin/env python
# ==============================================================
#
# SAMPLE SOURCE CODE - SUBJECT TO THE TERMS OF SAMPLE CODE LICENSE AGREEMENT,
# http://software.intel.com/en-us/articles/intel-sample-source-code-license-agreement/
#
# Copyright 2016 Intel Corporation
#
# THIS FILE IS PROVIDED "AS IS" WITH NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, NON-INFRINGEMENT OF INTELLECTUAL PROPERTY RIGHTS.
#
# =============================================================

#  Content:
#     Intel(R) Data Analytics Acceleration Library samples
# ******************************************************************************

import datetime
import os
import platform
import sys
from subprocess import Popen, PIPE

mpi_samples = [
    'svd_fast_distributed_mpi',
    'qr_fast_distributed_mpi',
    'implicit_als_csr_distributed_mpi',
    'linear_regression_norm_eq_distributed_mpi',
    'linear_regression_qr_distributed_mpi',
    'pca_correlation_dense_distributed_mpi',
    'pca_correlation_csr_distributed_mpi',
    'pca_svd_distributed_mpi',
    'covariance_dense_distributed_mpi',
    'covariance_csr_distributed_mpi',
    'multinomial_naive_bayes_dense_distributed_mpi',
    'multinomial_naive_bayes_csr_distributed_mpi',
    'low_order_moments_csr_distributed_mpi',
    'low_order_moments_dense_distributed_mpi',
]

IS_WIN = platform.system() == 'Windows'


def main(argv):
    if len(argv) > 2 or (len(argv) == 2 and argv[1] not in mpi_samples):
        show_help()

    samples_to_run = mpi_samples if len(argv) < 2 else [argv[1]]

    for sample in samples_to_run:
        file_path = os.path.join('sources', "{}.py".format(sample))
        if IS_WIN:
            cmd = ['mpiexec',  '-n', '4', '-localonly', 'python', file_path]
        else:
            cmd = ['mpirun', '-n', '4', '-ppn', '1', 'python', file_path]
        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        output, err = p.communicate()

        time = datetime.datetime.now().time().isoformat()
        sample_res_dir = os.path.join('_results', sample)

        if not os.path.isdir(sample_res_dir):
            os.makedirs(sample_res_dir)

        res_file = os.path.join(sample_res_dir, "{}.res".format(sample))

        with open(res_file, 'w') as f:
            if p.returncode == 0:
                f.write(output.decode('ascii') + '\n')
                status = 'PASSED'
            else:
                f.write("Error occurred: {}\n".format(err.decode('ascii')))
                status = 'FAILED'

        print("{} {}: {}".format(time, status, sample))

def show_help():
    help = "Usage: python launcher.py [sample]\n"
    help += "\tsample - Optional sample name. All samples will be run if absent."
    print(help)
    sys.exit(1)

if __name__ == '__main__':
    sys.exit(main(sys.argv))
